﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Xml;

using DsShared;
using DsUploader.model;

using Ionic.Zip;

namespace DsUploader.behaviour
{
	class Uploader
	{
		private static string RELEASE_NOTES_XML = "release_notes.xml";

		private NetworkCredential Credential = null;
        private DsShared.Status Status = null;

		public Uploader(NetworkCredential credential, DsShared.Status status)
		{
			Credential = credential;
            Status = status;
		}

		public void on(App a)
		{
			Console.WriteLine("upload app " + a.Local);
			try
			{
				string version = getVersion(a.Local);
				Console.WriteLine("\tversion " + version);

				// Prepare
				string fn = CreateZipFile(a, version);

				// Upload
				Upload(a, fn);

				// Cleanup
				File.Delete(fn);

				Console.WriteLine("\tfinished");
			}
			catch (SystemException ex)
			{
				Console.WriteLine("Error: " + ex.Message);
			}
		}

		// Get the version number for the folder, which
		// is contained in a release_notes.xml.
		private string getVersion(string folder)
		{
			string file = Path.Combine(folder, RELEASE_NOTES_XML);
			if (!File.Exists(file))
			{
				throw new UploaderException("Folder " + folder + " is missing file " + RELEASE_NOTES_XML);
			}

			XmlTextReader reader = new XmlTextReader(file);

			// Just skip xmls that are the wrong type
			if (reader == null || !reader.Read() || reader.Name != "release_notes")
			{
				throw new UploaderException("Folder " + folder + " is missing release_notes.xml");
			}

			// Get the first entry's version
			while (reader.Read())
			{
				if (reader.NodeType == XmlNodeType.Element && reader.Name == "release")
				{
					while (reader.MoveToNextAttribute())
					{
						if (reader.Name == "version")
						{
							return reader.Value;
						}
					}
					throw new UploaderException("File " + file + " is missing version attribute");
				}
			}
			throw new UploaderException("File " + file + " is missing <release> entry");
		}

		// Create the temporary zip file
		private string CreateZipFile(App a, string version)
		{
			string fn = Path.Combine(Path.GetTempPath(), DsShared.Util.getFilenameFromVersion(version) + ".zip");
			if (File.Exists(fn))
			{
				File.Delete(fn);
			}

			// Make sure that the extracted file creates the root folder name
			char[] charSeparators = new char[] { Path.DirectorySeparatorChar };
			string[] fullPath = a.Local.Split(charSeparators, StringSplitOptions.RemoveEmptyEntries);
			string root = fullPath[fullPath.Length-1];
			if (string.IsNullOrEmpty(root))
			{
				throw new DsShared.DsException("Error uploading item without a path leaf (" + a.Local + ")");
			}

			// Zip the folder and save it to a temporary file
			using (ZipFile zip = new ZipFile())
			{
				zip.CompressionLevel= Ionic.Zlib.CompressionLevel.BestCompression;
				zip.AddDirectory(a.Local, root);
				zip.Save(fn);
			}
			return fn;
		}

		// Do the actual file upload
		private void Upload(App a, string localZipFile)
		{
			// Find the target directory, make sure it exists
			NetworkLocation loc = new NetworkLocation(a.Remote);
			loc.Verify(Credential);
			loc.FtpUploadFile(Credential, localZipFile, Status);
		}
	}
	
	class UploaderException : System.Exception
	{
		public UploaderException() : base() { }
		public UploaderException(string message) : base(message) { }
		public UploaderException(string message, System.Exception inner) : base(message, inner) { }
	}

}
