﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Xml;

namespace DsShared
{
    class NetUtil
    {
        // Global network credentials.  Probably read from a file at some point.
        static public NetworkCredential Credential = new NetworkCredential("user", "password");
        // The target host for this app.
        public static string HOST = "";
        // The root directory that I perform remote operations in
        public static string ROOT = "";

        // Some networks require a passive connection.
        private static bool USE_PASSIVE = false;
        // It's possible someone will require SSL, but currently the server doesn't support it
        private static bool ENABLE_SSL = false;

        // Standardize on the backslash as the path separator for remote files.
        private static char[] back_char = { '/' };
        public static string BACK_SZ = new string(back_char);

        private static int BUFFER_SIZE = 8192;

        private static string COMPLETE_SZ = "\tcomplete: ";

		public static void initialize()
		{
			XmlTextReader reader = new XmlTextReader("settings.xml");

			// Just skip xmls that are the wrong type
			if (!reader.Read() || reader.Name != "settings")
			{
				throw new System.Exception("Missing the settings.xml file.  It should be located in the same directory as the application.");
			}
			string user = "";
			string password = "";
			while (reader.Read())
			{
				switch (reader.NodeType)
				{
					case XmlNodeType.Element:
						if (reader.Name == "host")
						{
							HOST = reader.ReadElementContentAsString();
						}
						else if (reader.Name == "path")
						{
							ROOT = reader.ReadElementContentAsString();
						}
						else if (reader.Name == "user")
						{
							user = reader.ReadElementContentAsString();
						}
						else if (reader.Name == "password")
						{
							password = reader.ReadElementContentAsString();
						}
						else if (reader.Name == "use_passive")
						{
							USE_PASSIVE = (string.Compare(reader.ReadElementContentAsString(), 0, "t", 0, 1, true) == 0);
						}
						else if (reader.Name == "use_ssl")
						{
							ENABLE_SSL = (string.Compare(reader.ReadElementContentAsString(), 0, "t", 0, 1, true) == 0);
						}
						break;
				}
			}
			if (string.IsNullOrEmpty(user) || string.IsNullOrEmpty(password) || string.IsNullOrEmpty(HOST))
			{
				throw new DsException("Incomplete settings (user=" + user + " passowrd=" + password + " host=" + HOST);
			}
			Credential = new NetworkCredential(user, password);
		}

        // Get the contents of a remote directory.  Remote should be
        // in the form ((host)/(path), i.e. "downstream.com/release"
        // If leaf is false, then the reply will contain the full directory path.
        public static string[] FtpGetRemoteDirectoryContents(NetworkCredential credential, string remote, bool leaf = true)
        {
            FtpWebRequest reqFTP;
            reqFTP = (FtpWebRequest)FtpWebRequest.Create(new Uri("ftp://" + remote));
            SetupFtpWebRequest(reqFTP);
            reqFTP.Credentials = credential;
            reqFTP.Method = WebRequestMethods.Ftp.ListDirectory;
            using (WebResponse response = reqFTP.GetResponse())
            {
                using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                {
                    List<string> directories = new List<string>();

                    string line = reader.ReadLine();
                    while (line != null)
                    {
                        if (leaf)
                        {
                            int last = line.LastIndexOf('/');
                            if (last != -1)
                            {
                                line = line.Substring(last + 1, line.Length - last - 1);
                            }
                        }
                        directories.Add(line);
                        line = reader.ReadLine();
                    }
                    return directories.ToArray();
                }
            }
        }

        // Create a remote directory.  The parent should exist.  Remote should be
        // in the form ((host)/(path), i.e. "downstream.com/release"
        public static void FtpCreateRemoteDirectory(NetworkCredential credential, string remote)
        {
            // Keep those slashes consist with the external world!
            string path = "ftp://" + remote.Replace(@"\", @"/");
            WebRequest request = WebRequest.Create(new Uri(path));
            request.Method = WebRequestMethods.Ftp.MakeDirectory;
            request.Credentials = credential;
            using (var resp = (FtpWebResponse)request.GetResponse())
            {
                if (resp.StatusCode != FtpStatusCode.PathnameCreated)
                {
                    throw new DsException("FtpCreateRemoteDirectory failed created (" + path + ")");
                }
            }
        }

        // Ge the size of a remote file, if possible.  Answer -1 if not.
        // Remote should be in the form ((host)/(path), i.e. "downstream.com/release"
        public static long FtpGetRemoteFileSize(NetworkCredential credential, string remote)
        {
            // Keep those slashes consist with the external world!
            remote = "ftp://" + remote.Replace(@"\", @"/");

            long fileSize = -1;
            try
            {
                FtpWebRequest request = (FtpWebRequest)FtpWebRequest.Create(remote);
                SetupFtpWebRequest(request);
                request.Credentials = credential;
                request.Method = WebRequestMethods.Ftp.GetFileSize;
                using (FtpWebResponse loginresponse = (FtpWebResponse)request.GetResponse())
                {
                    FtpWebResponse respSize = (FtpWebResponse)request.GetResponse();
                    respSize = (FtpWebResponse)request.GetResponse();
                    return respSize.ContentLength;
                }
            }
            catch (System.Exception)
            {
            }
            return fileSize;
        }

        // Download the source remote file to the local destination
        // Remote should be in the form ((host)/(path), i.e. "downstream.com/release"
        public static void FtpDownloadFile(NetworkCredential credential, string remote, string local, Status status)
        {
            long sum = 0;
            long total = FtpGetRemoteFileSize(credential, remote);
            string sizeError = null;
            if (total <= 0)
            {
                sizeError = " (Can't get file size)";
                total = -1;
            }
            // Keep those slashes consist with the external world!
            remote = "ftp://" + remote.Replace(@"\", @"/");

//            Console.WriteLine("DOWNLOAD=" + remote);
            // Get the object used to communicate with the server.
            FtpWebRequest request = (FtpWebRequest)WebRequest.Create(remote);
            SetupFtpWebRequest(request);
            request.Method = WebRequestMethods.Ftp.DownloadFile;
            request.Credentials = credential;

            using (FtpWebResponse response = (FtpWebResponse)request.GetResponse())
            {
                using (Stream responseStream = response.GetResponseStream())
                {
                    using (FileStream writeStream = new FileStream(local, FileMode.Create))
                    {
                        WriteStatusComplete(status, COMPLETE_SZ, (float)sum / (float)total, false, sizeError);

                        byte[] buffer = new byte[BUFFER_SIZE];
                        int read = responseStream.Read(buffer, 0, BUFFER_SIZE);
                        while (read > 0)
                        {
                            writeStream.Write(buffer, 0, read);
                            read = responseStream.Read(buffer, 0, BUFFER_SIZE);

                            sum = sum + read;
                            WriteStatusComplete(status, COMPLETE_SZ, (float)sum / (float)total, true, sizeError);
                        }
                        // File size doesn't seem to quite exactly match up to how many bytes are eventually read
                        WriteStatusComplete(status, COMPLETE_SZ, 1.0f, true, sizeError);
                        status.WriteLine("");
                    }
                }
            }
        }

        // Upload the source local file to the remote destination.  The remote parent directory
        // needs to exist.  Remote should be in the form ((host)/(path), i.e. "downstream.com/release"
        public static void FtpUploadFile(NetworkCredential credential, string local, string remote, Status status)
        {
            // Keep those slashes consist with the external world!
            remote = "ftp://" + remote.Replace(@"\", @"/");
            byte[] buffer = new byte[BUFFER_SIZE];

            FtpWebRequest request = (FtpWebRequest)WebRequest.Create(remote);
            SetupFtpWebRequest(request);
            request.Method = WebRequestMethods.Ftp.UploadFile;
            request.Credentials = credential;
            using (Stream ftp = request.GetRequestStream())
            {
                using (FileStream file = File.OpenRead(local))
                {
                    long sum = 0;
                    long total = file.Length;
                    int read = 0;
                    WriteStatusComplete(status, COMPLETE_SZ, (float)sum / (float)total, false, null);
                    do
                    {
                        read = file.Read(buffer, 0, BUFFER_SIZE);
                        ftp.Write(buffer, 0, read);

                        WriteStatusComplete(status, COMPLETE_SZ, (float)sum / (float)total, true, null);
                        sum = sum + read;
                    }
                    while (read != 0);
                }
                status.WriteLine("");
            }
            using (FtpWebResponse response = (FtpWebResponse)request.GetResponse())
            {
                if (response.StatusCode != FtpStatusCode.ClosingData)
                {
                    throw new DsException("Unknown error uploading to " + remote);
                }
            }
        }

        private static void SetupFtpWebRequest(FtpWebRequest ftp)
        {
            ftp.UseBinary = true;
            ftp.Proxy = null;
            ftp.KeepAlive = false;
            ftp.UsePassive = USE_PASSIVE;
            ftp.EnableSsl = ENABLE_SSL;
        }

        private static void WriteStatusComplete(Status status, string prefix, float unitAmount, bool replace, string error)
        {
            if (!replace)
            {
                if (string.IsNullOrEmpty(error))
                {
                    status.Write(prefix + " {0,-3:F2}%                                               ", +(unitAmount * 100.0));
                }
                else
                {
                    status.Write(prefix + error);
                }
            }
            else
            {
                if (string.IsNullOrEmpty(error))
                {
                    status.Replace(prefix + " {0,-3:F2}%                                               ", +(unitAmount * 100.0));
                }
                else
                {
                    status.Replace(prefix + error);
                }
            }
        }
    }
}
