﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;

namespace DsShared
{
    // Representation of a network location, in decomposed
    // parts (a host and a series of directories).
    class NetworkLocation
    {
        public string Host;
        public string[] Directory;

        // Making sure paths have the right separator is important for
        // some of the system calls
        private static char[] forward_char = { '\\' };
        private static char[] back_char = { '/' };
        private static char[] sep_char = { Path.DirectorySeparatorChar };
        private static string FORWARD_SZ = new string(forward_char);
        private static string BACK_SZ = new string(back_char);
        private static string SEP_SZ = new string(sep_char);

        // Assume the default host, decompose the path
        public NetworkLocation(string path)
        {
            Host = NetUtil.HOST;
            Decompose(path);
        }

        // Extract the host name from the path and
        // decompose the rest into directories.
        public NetworkLocation(string host, string path)
        {
            Host = host;
            Decompose(path);
        }

        // Verify that my remote path exists, creating if necessary
        public void Verify(NetworkCredential credential)
        {
            string path = Host;
            foreach (string d in Directory)
            {
                // If this part doesn't exist in my previous directory, create it
                string[] ans = DsShared.NetUtil.FtpGetRemoteDirectoryContents(credential, path);
                // Increment the remote location
                path = path + BACK_SZ + d;
                if (!Contains(ans, d))
                {
                    DsShared.NetUtil.FtpCreateRemoteDirectory(credential, path);
                }
            }
        }

        // Upload a file to the path I represent
        public void FtpUploadFile(NetworkCredential credential, string localFile, Status status)
        {
            string remoteFile = ComposePath();
            remoteFile = remoteFile + BACK_SZ + Path.GetFileName(localFile);
            DsShared.NetUtil.FtpUploadFile(credential, localFile, remoteFile, status);
        }

        private void Decompose(string path)
        {
            path = Path.Combine(NetUtil.ROOT, path);
            // Keep those slashes consist with the local file system,
            // so the split works.
            path = path.Replace(FORWARD_SZ, SEP_SZ);
            path = path.Replace(BACK_SZ, SEP_SZ);
            Directory = path.Split(Path.DirectorySeparatorChar);
        }

        private bool Contains(string[] list, string s)
        {
            foreach (string cmp in list)
            {
                if (cmp == s)
                {
                    return true;
                }
            }
            return false;
        }

        // Answer my host and path composed as a single string
        public string ComposePath()
        {
            string ans = Host;
            foreach (string d in Directory)
            {
                ans = ans + BACK_SZ + d;
            }
            return ans;
        }
    }
}
